var path = require('path')
  , url = require('url')
  , send = require('send')
  , kibu = require('../kibu')
  , _ = kibu._;

exports = module.exports = function staticServer(options) {
  options = options || {};

  var _root
    , _redirect = false || options.redirect;

  if (typeof options.root === 'undefined' || !options.root) {
    throw new Error('staticServer middleware requires a root path');
  }

  _root = path.join(kibu('root'), options.root);

  return function staticServer(req, res, next) {

    if (req.method !== 'GET' && req.method !== 'HEAD') return next();

    var pathname = url.parse(req.url).pathname
      , pause = _.pause(req);

    function onDirectory() {
      if (!_redirect) return resume();
      var target = url.format(pathname + '/');
      res.statusCode = 303;
      res.setHeader('Location', target);
      res.end('Redirecting to ' + _.escape(target));
    }

    function onError(err) {
      if (err.status === 404) return resume();
      next(err);
    }

    function resume() {
      next();
      pause.resume();
    }

    send(req, pathname)
      .maxage(options.maxAge || 0)
      .root(_root)
      .index(options.index || 'index.html')
      .hidden(options.hidden)
      .on('error', onError)
      .on('directory', onDirectory)
      .pipe(res);
  };
};